@@ -4,9 +4,9 @@ gset_memberships <-
 function(x, filter = NULL)
 {
     ex <- if(is.null(filter))
-        sets:::.get_memberships
+        .get_memberships
     else
-        function(i) sets:::.get_memberships(i[filter])
+        function(i) .get_memberships(i[filter])
     if (is.tuple(x)) lapply(x, ex) else ex(x)
 }
 
@@ -15,16 +15,25 @@ function(x)
     as.set(.get_support(x))
 
 gset_core <-
-function(x)
-    as.set(.get_support(x)[sapply(.get_memberships(x), max) == 1])
+function(x, na.rm = FALSE)
+{
+    m <- as.double(sapply(.get_memberships(x), max) >= 1)
+    if (na.rm) m[is.na(m)] <- FALSE
+    .make_gset_from_support_and_memberships(.get_support(x), m)
+}
 
 gset_height <-
-function(x)
-    max(unlist(.get_memberships(x)))
+function(x, na.rm = FALSE)
+    max(unlist(.get_memberships(x)), na.rm = na.rm)
 
 gset_peak <-
-function(x)
-    as.set(.get_support(x)[sapply(.get_memberships(x), max) == gset_height(x)])
+function(x, na.rm = FALSE)
+{
+    m <- as.double(sapply(.get_memberships(x), max) ==
+                   gset_height(x, na.rm = TRUE))
+    if (na.rm) m[is.na(m)] <- FALSE
+    .make_gset_from_support_and_memberships(.get_support(x), m)
+}
 
 gset_charfun <-
 function(x)
@@ -32,17 +41,15 @@ function(x)
     if (!is.gset(x))
         stop("Argument 'x' must be a generalized set.")
     ret <- function(e) {
-        if (is_element(e))
-            e <- list(e)
-        ret <- .exact_match(e, x)
-        vals <- !is.na(ret)
-        if (any(vals))
-            ret[vals] <- .get_memberships(x)[ ret[vals] ]
+        if (missing(e)) return(.get_memberships(x))
+        if (is_element(e)) e <- list(e)
+        ret <- .memberships_for_support(x, e)
         if (is.list(ret) && (length(ret) < 2L))
-            ret <- ret[[1L]]
-        if (is.na(ret)) 0 else ret
+            ret[[1L]]
+        else
+            ret
     }
-    structure(ret, class = "gset_charfun")
+    .structure(ret, class = "gset_charfun")
 }
 
 gset_universe <-
@@ -70,9 +77,7 @@ cset_core <- gset_core
 
 cset_height <- gset_height
 
-cset_peak <-
-function(x)
-    as.set(.get_support(x)[sapply(.get_memberships(x), max) == cset_height(x)])
+cset_peak <- gset_peak
 
 cset_charfun <-
 function(x)
@@ -81,17 +86,15 @@ function(x)
         stop("Argument 'x' must be a customizable set.")
     matchfun <- cset_matchfun(x)
     ret <- function(e) {
-        if (is_element(e))
-            e <- list(e)
-        ret <- matchfun(e, x)
-        vals <- !is.na(ret)
-        if (any(vals))
-            ret[vals] <- .get_memberships(x)[ ret[vals] ]
+        if (missing(e)) return(.get_memberships(x))
+        if (is_element(e)) e <- list(e)
+        ret <- .memberships_for_support(x, e, matchfun)
         if (is.list(ret) && (length(ret) < 2L))
-            ret <- ret[[1L]]
-        ret
+            ret[[1L]]
+        else
+            ret
     }
-    structure(ret, class = "cset_charfun")
+    .structure(ret, class = "cset_charfun")
 }
 
 cset_universe <- gset_universe
@@ -154,7 +157,7 @@ function(x, value)
 .get_fuzzy_multi_memberships <-
 function(x)
 {
-    if (gset_is_multiset(x))
+    if (!gset_is_fuzzy_multiset(x))
         lapply(.get_memberships(x), function(i) gset(1L, i))
     else
         .get_memberships(x)
@@ -167,7 +170,7 @@ function(x)
     if (is.null(u))
         u <- sets_options("universe")
     if (!is.null(u))
-        as.set(eval(u))
+        as.set(if (is.interval(u)) as.numeric(u) else eval(u))
     else
         as.set(.get_support(x))
 }
@@ -190,7 +193,7 @@ function(x)
     else if (set_is_empty(x))
         return(0)
     else
-        max(.multiplicities(.get_memberships(x)))
+        max(.multiplicities(.get_memberships(x)), na.rm = TRUE)
 }
 
 .set_bound <-
@@ -225,7 +228,7 @@ function(x)
 {
     if (is.list(x))
         unlist(lapply(x, function(i) sum(.get_memberships(i))))
-    else if(any(x > 1))
+    else if(any(x > 1, na.rm = TRUE))
         x
     else
         rep(1L, length(x))
