@@ -4,9 +4,11 @@ function(..., na.rm = FALSE)
 {
     l <- list(...)
     if (.Generic == "sum")
-        return(Reduce(function(i, j) sum(i, as.numeric(j)), l, 0))
+        return(Reduce(function(i, j) sum(i, as.numeric(j), na.rm = na.rm),
+                      l, 0))
     else if (.Generic == "prod")
-        return(Reduce(function(i, j) prod(i, as.numeric(j)), l, 1))
+        return(Reduce(function(i, j) prod(i, as.numeric(j), na.rm = na.rm),
+                      l, 1))
     do.call(.Generic, c(na.rm = na.rm, do.call(set_union, l)))
 }
 
@@ -35,11 +37,19 @@ function(..., na.rm = FALSE)
 }
 
 mean.gset <-
-function(x, ...)
+function(x, ..., na.rm = FALSE)
 {
     if (gset_is_fuzzy_multiset(x))
         stop("Operation not defined for fuzzy multisets.")
-    weighted.mean(as.numeric(x), .get_memberships(x), ...)
+
+    v <- as.numeric(x)
+    m <- .get_memberships(x)
+    if (na.rm && any(nas <- is.na(m))) {
+        v <- v[!nas]
+        m <- m[!nas]
+    }
+
+    weighted.mean(v, m, na.rm = na.rm)
 }
 
 median.gset <-
@@ -47,10 +57,15 @@ function(x, na.rm = FALSE)
 {
     if (gset_is_fuzzy_multiset(x))
         stop("Operation not defined for fuzzy multisets.")
-    x <- if (gset_is_fuzzy_set(x))
+    x <- if (gset_is_fuzzy_set(x, na.rm = TRUE))
         as.numeric(x) * .get_memberships(x)
-    else
-        rep.int(as.numeric(x), times = .get_memberships(x))
+    else {
+        n <- as.numeric(x)
+        m <- .get_memberships(x)
+        n[is.na(m)] <- NA
+        m[is.na(m)] <- 1
+        rep.int(n, times = m)
+    }
     median(x, na.rm = na.rm)
 }
 
@@ -68,8 +83,8 @@ function(..., na.rm = FALSE)
 }
 
 mean.cset <-
-function(x, ...)
-    mean.gset(x, ...)
+function(x, ..., na.rm = FALSE)
+    mean.gset(x, ..., na.rm = na.rm)
 
 median.cset <-
 function(x, na.rm = FALSE)
